@@ -1,15 +1,10 @@
-#!/usr/bin/env python
-# -*- coding: utf-8 -*-
 
 import os
 import time
+from collections import OrderedDict
+from io import BytesIO
 
-from translate.storage import po
-from translate.storage import poxliff
-from translate.storage import poheader
-from translate.misc.dictutils import ordereddict
-from translate.misc import wStringIO
-from translate.lang.team import guess_language
+from translate.storage import po, poheader, poxliff
 
 
 def test_parseheaderstring():
@@ -20,13 +15,13 @@ this item must get ignored because there is no colon sign in it
 item3: three
 '''
     d = poheader.parseheaderstring(source)
-    print type(d)
-    assert type(d) == ordereddict
+    print(type(d))
     assert len(d) == 3
     assert d['item1'] == 'one'
     assert d['item2'] == 'two:two'
     assert d['item3'] == 'three'
 
+
 def test_update():
     '''test the update function'''
     # do we really add nothing if add==False ?
@@ -37,37 +32,39 @@ def test_update():
     assert len(d) == 1
     assert d['Test'] == 'hello'
     # do we really update ?
-    d = poheader.update({'Test':'hello'}, add=True, Test='World')
+    d = poheader.update({'Test': 'hello'}, add=True, Test='World')
     assert len(d) == 1
     assert d['Test'] == 'World'
     # does key rewrite work ?
     d = poheader.update({}, add=True, test_me='hello')
     assert d['Test-Me'] == 'hello'
     # is the order correct ?
-    d = ordereddict()
+    d = OrderedDict()
     d['Project-Id-Version'] = 'abc'
     d['POT-Creation-Date'] = 'now'
     d = poheader.update(d, add=True, Test='hello', Report_Msgid_Bugs_To='bugs@list.org')
-    assert d.keys()[0] == "Project-Id-Version"
-    assert d.keys()[1] == "Report-Msgid-Bugs-To"
-    assert d.keys()[2] == "POT-Creation-Date"
-    assert d.keys()[3] == "Test"
+    assert list(d.keys()) == [
+        "Project-Id-Version", "Report-Msgid-Bugs-To", "POT-Creation-Date",
+        "Test",
+    ]
 
 
 def poparse(posource):
     """helper that parses po source without requiring files"""
-    dummyfile = wStringIO.StringIO(posource)
+    dummyfile = BytesIO(posource.encode())
     return po.pofile(dummyfile)
 
+
 def poxliffparse(posource):
     """helper that parses po source into poxliffFile"""
     poxli = poxliff.PoXliffFile()
     poxli.parse(posource)
     return poxli
 
+
 def check_po_date(datestring):
     """Check the validity of a PO date.
-    
+
     The datestring must be in the format: 2007-06-08 10:08+0200
     """
 
@@ -88,6 +85,7 @@ def check_po_date(datestring):
     # Check that the date can be parsed
     assert type(time.strptime(datestring, date_format)) == time.struct_time
 
+
 def test_po_dates():
     pofile = po.pofile()
     headerdict = pofile.makeheaderdict(po_revision_date=True)
@@ -95,15 +93,14 @@ def test_po_dates():
     check_po_date(headerdict["PO-Revision-Date"])
 
     headerdict = pofile.makeheaderdict(pot_creation_date=time.localtime(),
-        po_revision_date=time.localtime())
+                                       po_revision_date=time.localtime())
     check_po_date(headerdict["POT-Creation-Date"])
     check_po_date(headerdict["PO-Revision-Date"])
 
-def test_timezones():
-    pofile = po.pofile()
 
+def test_timezones():
     # The following will only work on Unix because of tzset() and %z
-    if time.__dict__.has_key('tzset'):
+    if 'tzset' in time.__dict__:
         os.environ['TZ'] = 'Asia/Kabul'
         time.tzset()
         assert time.timezone == -16200
@@ -122,28 +119,22 @@ def test_timezones():
         # Typically "+0200"
         assert poheader.tzstring() == time.strftime("%z")
 
-        os.environ['TZ'] = 'Africa/Windhoek'
-        time.tzset()
-        assert time.timezone == -3600
-        # Typically "+0100"
-        # For some reason python's %z doesn't know about Windhoek DST
-        #assert poheader.tzstring() == time.strftime("%z")
-
         os.environ['TZ'] = 'UTC'
         time.tzset()
         assert time.timezone == 0
         # Typically "+0000"
         assert poheader.tzstring() == time.strftime("%z")
 
+
 def test_header_blank():
 
     def compare(pofile):
-        print pofile
+        print(pofile)
         assert len(pofile.units) == 1
         header = pofile.header()
         assert header.isheader()
         assert not header.isblank()
-    
+
         headeritems = pofile.parseheader()
         assert headeritems["Project-Id-Version"] == "PACKAGE VERSION"
         assert headeritems["Report-Msgid-Bugs-To"] == ""
@@ -155,8 +146,7 @@ def test_header_blank():
         assert headeritems["Content-Type"] == "text/plain; charset=UTF-8"
         assert headeritems["Content-Transfer-Encoding"] == "8bit"
         assert headeritems["Plural-Forms"] == "nplurals=INTEGER; plural=EXPRESSION;"
-    
-    
+
     """test header functionality"""
     posource = r'''# other comment\n
 msgid ""
@@ -179,9 +169,9 @@ msgstr ""
 ##
 ##    poxliffsource = r'''<?xml version="1.0" encoding="utf-8"?>
 ##<xliff version="1.1" xmlns="urn:oasis:names:tc:xliff:document:1.1">
-## 
+##
 ##<file datatype="po" original="test.po" source-language="en-US"><body><trans-unit approved="no" id="1" restype="x-gettext-domain-header" xml:space="preserve"><source>Project-Id-Version: PACKAGE VERSION
-##Report-Msgid-Bugs-To: 
+##Report-Msgid-Bugs-To:
 ##POT-Creation-Date: 2006-03-08 17:30+0200
 ##PO-Revision-Date: YEAR-MO-DA HO:MI+ZONE
 ##Last-Translator: FULL NAME <ph id="1">&lt;EMAIL@ADDRESS&gt;</ph>
@@ -191,7 +181,7 @@ msgstr ""
 ##Content-Transfer-Encoding: 8bit
 ##Plural-Forms: nplurals=INTEGER; plural=EXPRESSION;
 ##</source><target>Project-Id-Version: PACKAGE VERSION
-##Report-Msgid-Bugs-To: 
+##Report-Msgid-Bugs-To:
 ##POT-Creation-Date: 2006-03-08 17:30+0200
 ##PO-Revision-Date: YEAR-MO-DA HO:MI+ZONE
 ##Last-Translator: FULL NAME <ph id="1">&lt;EMAIL@ADDRESS&gt;</ph>
@@ -215,18 +205,19 @@ msgstr ""
 '''
     for colon in ("", ";"):
         pofile = poparse(posource % colon)
-        print pofile
+        print(pofile)
         assert len(pofile.units) == 1
         header = pofile.units[0]
         assert header.isheader()
         assert not header.isblank()
 
-        headeritems = pofile.parseheader()
+        pofile.parseheader()
         nplural, plural = pofile.getheaderplural()
         assert nplural == "2"
         assert plural == "(n != 1)"
 ##    TODO: add the same test for PoXliffFile
 
+
 def test_plural_equation_across_lines():
     """test that we work if the plural equation spans more than one line"""
     posource = r'''msgid ""
@@ -235,18 +226,19 @@ msgstr ""
 "10<=4 && (n%100<10 || n%100>=20) ? 1 : 2);\n"
 '''
     pofile = poparse(posource)
-    print pofile
+    print(pofile)
     assert len(pofile.units) == 1
     header = pofile.units[0]
     assert header.isheader()
     assert not header.isblank()
 
-    headeritems = pofile.parseheader()
+    pofile.parseheader()
     nplural, plural = pofile.getheaderplural()
     assert nplural == "3"
     assert plural == "(n%10==1 && n%100!=11 ? 0 : n%10>=2 && n%10<=4 && (n%100<10 || n%100>=20) ? 1 : 2)"
 ##    TODO: add the same test for PoXliffFile
 
+
 def test_updatecontributor():
     """Test that we can update contributor information in the header comments."""
     posource = r'''msgid ""
@@ -255,15 +247,38 @@ msgstr ""
 '''
     pofile = poparse(posource)
     pofile.updatecontributor("Grasvreter")
-    assert "# Grasvreter, 20" in str(pofile)
+    assert "# Grasvreter, 20" in bytes(pofile).decode('utf-8')
 
     pofile.updatecontributor("Koeivreter", "monster@grasveld.moe")
-    assert "# Koeivreter <monster@grasveld.moe>, 20" in str(pofile)
+    assert "# Koeivreter <monster@grasveld.moe>, 20" in bytes(pofile).decode('utf-8')
 
     pofile.header().addnote("Khaled Hosny <khaledhosny@domain.org>, 2006, 2007, 2008.")
     pofile.updatecontributor("Khaled Hosny", "khaledhosny@domain.org")
-    print str(pofile)
-    assert "# Khaled Hosny <khaledhosny@domain.org>, 2006, 2007, 2008, %s." % time.strftime("%Y") in str(pofile)
+    print(bytes(pofile))
+    assert "# Khaled Hosny <khaledhosny@domain.org>, 2006, 2007, 2008, %s." % time.strftime("%Y") in bytes(pofile).decode('utf-8')
+
+
+def test_updatecontributor_header():
+    """Test preserving empty lines in comments"""
+    posource = r'''# Japanese translation of ibus.
+# Copyright (C) 2015-2019 Takao Fujiwara <takao.fujiwara1@gmail.com>
+# This file is distributed under the same license as the ibus package.
+#
+# Translators:
+# Takao Fujiwara <takao.fujiwara1@gmail.com>, 2012
+msgid ""
+msgstr ""
+"MIME-Version: 1.0"
+'''
+    pofile = poparse(posource)
+
+    # Add contributor
+    pofile.updatecontributor("Grasvreter")
+
+    # Manually build expected output
+    expected = posource.replace("msgid", "# Grasvreter, %s.\nmsgid" % time.strftime("%Y"))
+    assert bytes(pofile).decode('utf-8') == expected
+
 
 def test_language():
     """Test that we can get a language from the relevant headers."""
@@ -273,7 +288,7 @@ msgstr ""
 '''
 
     pofile = poparse(posource)
-    assert pofile.gettargetlanguage() == None
+    assert pofile.gettargetlanguage() is None
 
     posource += '"Language-Team: translate-discuss-af@lists.sourceforge.net\\n"\n'
     pofile = poparse(posource)
@@ -286,3 +301,32 @@ msgstr ""
     posource += '"Language: fr_CA\\n"\n'
     pofile = poparse(posource)
     assert pofile.gettargetlanguage() == 'fr_CA'
+
+
+def test_project():
+    """Test that we can get a project from the relevant headers."""
+    posource = r'''msgid ""
+msgstr ""
+"MIME-Version: 1.0\n"
+'''
+
+    pofile = poparse(posource)
+    assert pofile.getprojectstyle() is None
+
+    posource += '"X-Accelerator-Marker: ~\\n"\n'
+    pofile = poparse(posource)
+    assert pofile.getprojectstyle() == 'openoffice'
+
+    posource += '"Report-Msgid-Bugs-To: http://bugzilla.gnome.org/enter_bug.cgi?product=system-\\n"\n'
+    pofile = poparse(posource)
+    assert pofile.getprojectstyle() == 'gnome'
+
+    posource += '"X-Project-Style: drupal\\n"\n'
+    pofile = poparse(posource)
+    assert pofile.getprojectstyle() == 'drupal'
+
+    pofile.setprojectstyle('kde')
+    assert pofile.getprojectstyle() == 'kde'
+
+    pofile.setprojectstyle('complete-rubbish')
+    assert pofile.getprojectstyle() == 'complete-rubbish'
